/*
 * This file is a part of VyQChat.
 *
 * Copyright (C) 2002-2004 Pawel Stolowski <yogin@linux.bydg.org>
 *
 * VyQChat is free software; you can redestribute it and/or modify it
 * under terms of GNU General Public License by Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY. See GPL for more details.
 */

#ifndef __SETTINGS_H
#define __SETTINGS_H

#include "sound.h"
#include "vcprotocol.h"
#include "uuid.h"
#include "history.h"
#include <qstring.h>
#include <qcolor.h>
#include <qsettings.h>

class SettingsDialog;
class UUID;

class Settings
{
	private:
		UUID uuid;
		QString icontheme;
		QString nick, def_nick;
		QString hostname, def_hostname;
		QString ip, def_ip;
		unsigned int port, def_port;
		QString broadcast, def_broadcast, mcastgrp, def_mcastgrp;
		QString topic, def_topic;
		Sound::SoundType def_sound;
		int status;  //status on exit
		int refresh;
		QColor special_color;
		QColor normal_color;
		QColor own_color;
		int x, y, width, height, listwidth;
		Sound::SoundType sound;
		QString playcmd;
		QString snd[Sound::Dummy];
		bool sndp[Sound::Dummy]; //play-dont play flags
		QString def_snd[Sound::Dummy];
		QString def_playcmd;
		bool useutf;
		bool confirm_exit;
		bool hide_channels;
		bool regexp_on;
		bool toolbar_on;
		bool userslist_on;
		bool nickcombo_on;
		bool popup;
		bool nopopup_dnd;
		bool keep_status; //if last status should be restored on startup
		QString encoding;
		QString version; //vyqchat version for detecting older config file

		QSettings cfg;

		void setUUID(const UUID u) { uuid = u; cfg.writeEntry("/General/UUID", uuid.asString()); }
		void setVersion(const QString &v) { cfg.writeEntry("/General/Version", this->version = v); }
	
		const QString getSampleKey(Sound::Sfx s)
		{
			QString key;
			switch (s)
			{
				case Sound::Chatline: key = "/Chatline"; break;
				case Sound::Message: key = "/Message"; break;
				case Sound::JoinChannel: key = "/Join"; break;
				case Sound::LeaveChannel: key = "/Leave"; break;
				case Sound::Beep: key = "/Beep"; break;
				case Sound::RegexpMatch: key = "RegexpMatch"; break;
				default: break;
			}
			return key;
		}

	public:
		Settings();
		~Settings();
		bool getDefaults();
		bool load();

		//
		// Get methods
		const UUID Settings::getUUID()     { return uuid; }
		const QString& getIconThemeName()  { return icontheme; }
		const QString& getNick()           { return nick; }
		const QString& getBroadcast()      { return broadcast; }
		const QString& getIP()             { return ip; }
		const QString& getHostname()       { return hostname; }
		const QString& getMulticastGroup() { return mcastgrp; }
		const unsigned int getPort()       { return port; }
		int getStatus()                    { return status; }
		bool getKeepStatus()               { return keep_status; }
		bool getRegexpOn()                 { return regexp_on; }
		bool getTooolbarOn()               { return toolbar_on; }
		bool getHideChannels()             { return hide_channels; }
		bool getUsersListOn()              { return userslist_on; }
		bool getNickComboOn()              { return nickcombo_on; }
		int getUsersListWidth()            { return listwidth; }
		int getWidth()                     { return width; }
		int getHeight()                    { return height; }
		int getX()                         { return x; }
		int getY()                         { return y; }
		int getRefresh()                   { return refresh; }
		const QString& getVersion()        { return version; }
		const QString& getEncoding()       { return encoding; }
		bool getUseUTF()                   { return useutf; }
		const QString& getTopic()          { return topic; }
		bool getConfirmExit()              { return confirm_exit; }
		bool getPopupsOn()                 { return popup; }
		bool getNoDNDpopups()              { return nopopup_dnd; }
		bool isSoundEnabled(Sound::Sfx)    { return true; }
		const QString& getPlayCommand()    { return playcmd; }
		Sound::SoundType getSoundType()    { return sound; }
		bool getSampleEnabled(Sound::Sfx i) { return sndp[i]; }
		const QString& getSampleFileName(Sound::Sfx i) { return snd[i]; }
		History getDNDhistory()       const  { return cfg.readListEntry("/History/dnd"); }
		History getOfflineHistory()   const  { return cfg.readListEntry("/History/offline"); }
		History getAwayHistory()      const  { return cfg.readListEntry("/History/away"); }
		History getNicksHistory()     const  { return cfg.readListEntry("/History/nicks"); }
		History getRegexpHistory()    const  { return cfg.readListEntry("/History/regexp"); }

		//
		// Set methods
		void setIconThemeName(const QString &name) { cfg.writeEntry("/Look/IconTheme", this->icontheme = name); }
		void setNick(const QString &nick) { cfg.writeEntry("/Network/Nick", this->nick = nick); }
		void setHostname(const QString &hostname) { cfg.writeEntry("/Network/Hostname", this->hostname = hostname); }
		void setIP(const QString &ip) { cfg.writeEntry("/Network/IP", this->ip = ip); }
		void setPort(unsigned int p) { cfg.writeEntry("/Network/Port", static_cast<int>(this->port = p)); }
		void setBroadcast(const QString &ip) { cfg.writeEntry("/Network/Broadcast", this->broadcast = ip); }
		void setMulticast(const QString &ip) { cfg.writeEntry("/Network/MulticastGroup", this->mcastgrp = ip); }
		void setRefresh(const int t) { cfg.writeEntry("/Network/Refresh", this->refresh = t); }
		void setGeometry(int x, int y, int w, int h)
		{
			cfg.beginGroup("/Window");
			cfg.writeEntry("/x", this->x = x);
			cfg.writeEntry("/y", this->y = y);
			cfg.writeEntry("/Width", this->width = w);
			cfg.writeEntry("/Height", this->height = h);
			cfg.endGroup();
		}
		void setUsersListWidth(int w) { listwidth = w; }
		void setRegexpOn(bool f) { cfg.writeEntry("/General/RegExpOn", regexp_on = f); }
		void setToobarOn(bool f) { cfg.writeEntry("/General/ToolbarOn", toolbar_on = f); }
		void setUsersListOn(bool f) { cfg.writeEntry("/General/UsersListOn", userslist_on = f); }
		void setNickComboOn(bool f) { cfg.writeEntry("/General/NickComboOn", nickcombo_on = f); }
		void setTopic(const QString &t) { cfg.writeEntry("/General/Topic", topic = t); }
		void setConfirmExit(bool f) { cfg.writeEntry("/General/ConfirmExit", confirm_exit = f); }
		void setKeepStatus(bool f) { cfg.writeEntry("/General/KeepStatus", keep_status = f); }
		void setStatus(int s) { cfg.writeEntry("/General/Status", status = s); }
		void setHideChannels(bool f) { cfg.writeEntry("/Misc/HideChannels", hide_channels = f); }
		void setPopups(bool f) { cfg.writeEntry("/Misc/Popup", popup = f); }
		void setNoDNDpopups(bool f) { cfg.writeEntry("/Misc/NoPopupDND", nopopup_dnd = f); }
		void setEncodingName(const QString e) { cfg.writeEntry("/Misc/Encoding", encoding = e); }
		void setUseUTF(bool f) { cfg.writeEntry("/Misc/UTF", useutf = f); }
		void setSoundType(Sound::SoundType t) { cfg.writeEntry("/Sound/sound", sound = t); }
		void setSampleFileName(Sound::Sfx s, const QString &f)
		{
			QString key = getSampleKey(s);
			if (!key.isEmpty())
			{
				cfg.beginGroup("/Wavs");
				cfg.writeEntry(key, snd[s] = f);
				cfg.endGroup();
			}
		}
		void setSampleEnabled(Sound::Sfx s, bool f)
		{
			QString key = getSampleKey(s);
			if (!key.isEmpty())
			{
				cfg.beginGroup("/WavFlags");
				cfg.writeEntry(key, snd[s] = f);
				cfg.endGroup();
			}
		}
			
		void setDNDhistory(const History &s) { cfg.writeEntry("/History/dnd", s.getAll()); }
		void setAwayHistory(const History &s) { cfg.writeEntry("/History/away", s.getAll()); }
		void setOfflineHistory(const History &s) { cfg.writeEntry("/History/offline", s.getAll()); }
		void setNicksHistory(const History &s) { cfg.writeEntry("/History/nicks", s.getAll()); }
		void setRegexpHistory(const History &s) { cfg.writeEntry("/History/regexp", s.getAll()); }

};


#endif

